@@ -22,7 +22,7 @@ class shader_Background_t: public background_t
 	public:
 		shader_Background_t(shader_t* in);
 		//virtual ~shader_Background_t();
-		virtual color_t operator () (const vector3d_t &dir,renderState_t &state) const;
+		virtual color_t operator () (const vector3d_t &dir, renderState_t &state, bool filtered) const;
 		static background_t *factory(paramMap_t &,renderEnvironment_t &);
 	protected:
 		shader_t *input;
