@@ -13,7 +13,7 @@ class constBackground_t: public background_t
 {
 	public:
 		constBackground_t(const color_t &c) {color=c;};
-		virtual color_t operator () (const vector3d_t &dir,renderState_t &state)const {return color;};
+		virtual color_t operator() (const vector3d_t &dir,renderState_t &state, bool filtered=false) const { return color; }
 		virtual ~constBackground_t() {};
 		static background_t *factory(paramMap_t &,renderEnvironment_t &);
 	protected:
@@ -26,7 +26,7 @@ class sunskyBackground_t: public background_t
 	public:
 		sunskyBackground_t(const point3d_t dir, PFLOAT turb,
 			PFLOAT a_var, PFLOAT b_var, PFLOAT c_var, PFLOAT d_var, PFLOAT e_var);
-		virtual color_t operator () (const vector3d_t &dir,renderState_t &state) const;
+		virtual color_t operator() (const vector3d_t &dir, renderState_t &state, bool filtered=false) const;
 		virtual ~sunskyBackground_t() {};
 		static background_t *factory(paramMap_t &,renderEnvironment_t &);
 	protected:
