@@ -21,7 +21,7 @@
  */
 
 #include "arealight.h"
-#include <cmath>
+//#include <cmath>
 #include <cstdlib>
 
 using namespace std;
@@ -78,16 +78,16 @@ areaLight_t::areaLight_t(const point3d_t &a,const point3d_t &b,
 												CFLOAT inte,int fsam,bool dum):
 												points(nsam),jit(nsam),dummy(dum)
 {
-	samples=fillQuad(a,b,c,d,points,jit,nsam);
-	direction=(b-a)^(d-a);
+	samples = fillQuad(a, b, c, d, points, jit, nsam);
+	direction = (b-a)^(d-a);
 	direction.normalize();
-	color=col;
-	pow=inte;
-	from=(a+b+c+d)/4;
-	fsamples=fsam;
-	corner=a;
-	toX=b-a;
-	toY=d-a;
+	color = col;
+	pow = inte;
+	from = (a+b+c+d)*0.25;
+	fsamples = fsam;
+	corner = a;
+	toX = b-a;
+	toY = d-a;
 }
 
 color_t areaLight_t::illuminate(renderState_t &state,const scene_t &s,const surfacePoint_t sp,
@@ -118,11 +118,12 @@ color_t areaLight_t::illuminate(renderState_t &state,const scene_t &s,const surf
 	if (samples==1) {
 		point3d_t sampleP = corner + toX*ourRandom() + toY*ourRandom();
 		L = sampleP-sp.P();
-		if ((L*N)<0) return color_t(0.0);
+		if ((L*N)<0) { state.skipelement=oldorigin; return color_t(0.0); }
 		if (!s.isShadowed(state, sp, sampleP)) {
 			dir = L;
-			dir.normalize();
-			energy_t ene(dir, (pow*color)/(L*L));
+			CFLOAT LD2 = dir.normLenSqr();
+			LD2 = (LD2!=0) ? (1.0/LD2) : 1.0;
+			energy_t ene(dir, (pow*color)*LD2);
 			resul += sha->fromLight(state, sp, ene, eye);
 		}
 		state.skipelement = oldorigin;
@@ -135,17 +136,17 @@ color_t areaLight_t::illuminate(renderState_t &state,const scene_t &s,const surf
 		case PENUMBRA:
 			for(int i=0;i<samples;++i)
 			{
-				PFLOAT dish=ourRandom()-0.5;
-				PFLOAT disv=ourRandom()-0.5;
-				point3d_t sampleP=points[i]+jit[i].first*dish+jit[i].second*disv;
-				L=sampleP-sp.P();
-				if((L*N)<0) continue;
-				if(!s.isShadowed(state,sp,sampleP))
+				PFLOAT dish=ourRandom()-0.5, disv=ourRandom()-0.5;
+				point3d_t sampleP = points[i] + jit[i].first*dish + jit[i].second*disv;
+				L = sampleP-sp.P();
+				if ((L*N)<0) continue;
+				if (!s.isShadowed(state, sp, sampleP))
 				{
-					dir=L;
-					dir.normalize();
-					energy_t ene(dir, (pow*color)/(L*L));
-					resul=resul+sha->fromLight(state,sp,ene,eye);
+					dir = L;
+					CFLOAT LD2 = dir.normLenSqr();
+					LD2 = (LD2!=0) ? (1.0/LD2) : 1.0;
+					energy_t ene(dir, (pow*color)*LD2);
+					resul += sha->fromLight(state, sp, ene, eye);
 				}
 			}
 			state.skipelement=oldorigin;
@@ -153,11 +154,12 @@ color_t areaLight_t::illuminate(renderState_t &state,const scene_t &s,const surf
 		case LIGHT:
 			for(int i=0;i<samples;++i)
 			{
-				L=points[i]-sp.P();
-				dir=L;
-				dir.normalize();
-				energy_t ene(dir, (pow*color)/(L*L));
-				resul=resul+sha->fromLight(state,sp,ene,eye);
+				L = points[i]-sp.P();
+				dir = L;
+				CFLOAT LD2 = dir.normLenSqr();
+				LD2 = (LD2!=0) ? (1.0/LD2) : 1.0;
+				energy_t ene(dir, (pow*color)*LD2);
+				resul += sha->fromLight(state, sp, ene, eye);
 			}
 			state.skipelement=oldorigin;
 			return (plane_at*resul/ ((CFLOAT)samples));
