@@ -28,29 +28,30 @@ __BEGIN_YAFRAY
 #define WARNING cerr<<"[hemilight]: "
 #define INFO cerr<<"[hemilight]: "
 
-hemiLight_t::hemiLight_t(int nsam, const color_t &c, CFLOAT pwr, bool usebg, bool useqmc)
-	: samples(nsam), color(c), power(pwr), use_background(usebg), use_QMC(useqmc)
+hemiLight_t::hemiLight_t(int nsam, const color_t &c, CFLOAT pwr, PFLOAT mdist, bool usebg, bool useqmc)
+	: samples(nsam), color(c), power(pwr), maxdistance(mdist), use_background(usebg), use_QMC(useqmc)
 {
-  if (use_QMC) {
-    // two dim only
-    HSEQ = new Halton[2];
-    HSEQ[0].setBase(2);
-    HSEQ[1].setBase(3);
-  }
-  else {
-    // samples must be integer squared value for jittered sampling
-    int g = int(sqrt((float)samples));
-    g *= g;
-    if (g!=samples) {
-      cout << "Samples value changed from " << samples << " to " << g << endl;
-      samples = g;
-    }
-    grid = int(sqrt((float)samples));
-    gridiv = 1.0/PFLOAT(grid);
-    gridiv2pi = gridiv * 2.0 * M_PI;
-    HSEQ = NULL;
-  }
-  sampdiv = power / PFLOAT(samples);
+	if (use_QMC) {
+		// two dim only
+		HSEQ = new Halton[2];
+		HSEQ[0].setBase(2);
+		HSEQ[1].setBase(3);
+	}
+	else {
+		// samples must be integer squared value for jittered sampling
+		int g = int(sqrt((float)samples));
+		g *= g;
+		if (g!=samples) {
+			cout << "Samples value changed from " << samples << " to " << g << endl;
+			samples = g;
+		}
+		grid = int(sqrt((float)samples));
+		gridiv = 1.0/PFLOAT(grid);
+		gridiv2pi = gridiv * 2.0 * M_PI;
+		HSEQ = NULL;
+	}
+	//sampdiv = 2.0*power/(PFLOAT)samples;	// unif.hemi pdf=2
+	sampdiv = power/(PFLOAT)samples;
 }
 
 
@@ -63,40 +64,53 @@ color_t hemiLight_t::illuminate(renderState_t &state,const scene_t &sc, const su
 	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 
 	energy_t fake(N, color_t(1.0));
-	color_t guesscolor = sha->fromLight(state,sp, fake, eye);
-	CFLOAT guessf = maxAbsDiff(guesscolor, color_t(0.0));
-	if (guessf<0.05f) return totalcolor;
+	if (maxAbsDiff(sha->fromLight(state, sp, fake, eye), color_t(0.0))<0.05f) return totalcolor;
 
 	const void *oldorigin=state.skipelement;
 	state.skipelement=sp.getOrigin();
+
+	//CFLOAT totalocc = 0;
+	//vector3d_t avgdir(0, 0, 0);
 	for (int sm=0;sm<samples;sm++)
 	{
 		dir = getNext(N, sm, sp.NU(), sp.NV());
-		if (!sc.isShadowed(state,sp, dir))
+		CFLOAT occ = dir*N;
+		if ((occ>0) && (!((maxdistance>0) ?
+					sc.isShadowed(state, sp, sp.P()+maxdistance*dir) :
+					sc.isShadowed(state, sp, dir))))
 		{
+			
 			if (use_background)
-				totalcolor += sha->fromLight(state,sp, energy_t(dir, sc.getBackground(dir,state)), eye);
+				totalcolor += sc.getBackground(dir, state, true) * occ;
 			else
-				totalcolor += sha->fromLight(state,sp, energy_t(dir, color), eye);
+				totalcolor += color * occ;
+			
+			//totalocc += occ;
+			//avgdir += dir;
 		}
 	}
-	state.skipelement=oldorigin;
-	return totalcolor * sampdiv;
+	state.skipelement = oldorigin;
+	return sampdiv * totalcolor * sha->fromLight(state, sp, fake, eye);
+	/*
+	avgdir.normalize();
+	fake.dir = avgdir;
+	if (use_background) return sampdiv * totalocc * sha->fromLight(state, sp, fake, eye) * sc.getBackground(avgdir, state, true);
+	return sampdiv * totalocc * sha->fromLight(state, sp, fake, eye) * color;
+	*/
 }
 
-
 // returns new hemi vector with uniform distribution
 vector3d_t hemiLight_t::getNext(const vector3d_t &normal, int cursample, const vector3d_t &Ru, const vector3d_t &Rv) const
 {
-  PFLOAT z1, z2;
-  if (use_QMC) {
-    z1=HSEQ[0].getNext();  z2=HSEQ[1].getNext()*2.0*M_PI;
-  }
-  else {
-    z1 = (PFLOAT(cursample / grid) + ourRandom()) * gridiv;
-    z2 = (PFLOAT(cursample % grid) + ourRandom()) * gridiv2pi;
-  }
-  return (Ru*cos(z2) + Rv*sin(z2))*sqrt(1.0-z1*z1) + normal*z1;
+	PFLOAT z1, z2;
+	if (use_QMC) {
+		z1=HSEQ[0].getNext();  z2=HSEQ[1].getNext()*2.0*M_PI;
+	}
+	else {
+		z1 = (PFLOAT(cursample / grid) + ourRandom()) * gridiv;
+		z2 = (PFLOAT(cursample % grid) + ourRandom()) * gridiv2pi;
+	}
+	return (Ru*cos(z2) + Rv*sin(z2))*sqrt(1.0-z1*z1) + normal*z1;
 }
 
 light_t *hemiLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
@@ -104,6 +118,7 @@ light_t *hemiLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	color_t color;
 	CFLOAT power = 1.0;
 	int samples = 16;
+	PFLOAT mdist = -1;	// infinite default
 	bool use_background = false;
 	bool useqmc = false;
 
@@ -119,7 +134,9 @@ light_t *hemiLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 		samples = 1;
 	}
 	params.getParam("use_QMC", useqmc);
-	return new hemiLight_t(samples, color, power, use_background, useqmc);
+	
+	params.getParam("maxdistance", mdist);
+	return new hemiLight_t(samples, color, power, mdist, use_background, useqmc);
 }
 
 pluginInfo_t hemiLight_t::info()
