@@ -31,11 +31,11 @@ lightCache_t *lightcache=NULL;
 
 
 pathLight_t::pathLight_t(int nsam, CFLOAT pwr, int depth,int cdepth,bool uQ,
-		bool ca,PFLOAT casiz,CFLOAT thr,bool recal,bool di,bool shows,int grids,int ref)
+		bool ca,PFLOAT casiz,CFLOAT thr,bool recal,bool di,bool shows,int grids,int ref,
+		bool _occmode, PFLOAT occdist, bool _ignorms)
 		: samples(nsam), power(pwr), maxdepth(depth),maxcausdepth(cdepth),use_QMC(uQ),
 cache(ca),maxrefinement(ref),recalculate(recal),direct(di),show_samples(shows),
-gridsize(grids),threshold(thr)
-
+gridsize(grids),threshold(thr), occmode(_occmode), occ_maxdistance(occdist), ignorms(_ignorms)
 {
 	if(cache) 
 	{
@@ -49,30 +49,30 @@ gridsize(grids),threshold(thr)
 		dist_to_sample=casiz*0.1;
 	}
 
-  if (use_QMC) 
+	if (use_QMC) 
 	{
-    depth = (depth+1)*2;
-    HSEQ = new Halton[depth];
-    int base = 2;
-    for (int i=0;i<depth;i++) {
-      HSEQ[i].setBase(base);
-      base = nextPrime(base);
-    }
-  }
-  else 
+		depth = (depth+1)*2;
+		HSEQ = new Halton[depth];
+		int base = 2;
+		for (int i=0;i<depth;i++) {
+			HSEQ[i].setBase(base);
+			base = nextPrime(base);
+		}
+	}
+	else 
 	{
-    // samples must be integer squared value for jittered sampling
-    int g = int(sqrt((float)samples));
-    g *= g;
-    if (g!=samples) {
-      cout << "Samples value changed from " << samples << " to " << g << endl;
-      samples = g;
-    }
-    grid = int(sqrt((float)samples));
-    gridiv = 1.0/PFLOAT(grid);
-    HSEQ = NULL;
-  }
-  sampdiv = 1.0 / PFLOAT(samples);
+		// samples must be integer squared value for jittered sampling
+		int g = int(sqrt((float)samples));
+		g *= g;
+		if (g!=samples) {
+			cout << "Samples value changed from " << samples << " to " << g << endl;
+			samples = g;
+		}
+		grid = int(sqrt((float)samples));
+		gridiv = 1.0/PFLOAT(grid);
+		HSEQ = NULL;
+	}
+	sampdiv = 1.0 / PFLOAT(samples);
 
 	shadow_threshold=0.1;
 	desiredWeight=1.0/shadow_threshold;
@@ -95,7 +95,7 @@ void pathLight_t::init(scene_t &scene)
 		lightcache->setAspect(scene.getAspectRatio());
 		lightcache->startFill();
 		scene.setRepeatFirst();
-		devaluated=1.0;
+		devaluated = 1.0;
 	}
 	use_in_indirect=false;
 	scene.getPublishedData("globalPhotonMap",pmap);
@@ -107,12 +107,15 @@ void pathLight_t::init(scene_t &scene)
 color_t pathLight_t::illuminate(renderState_t &state,const scene_t &sc, const surfacePoint_t sp,
 				const vector3d_t &eye) const
 {
-	if(cache) 
+	if(cache)
 	{
-		if(lightcache->ready()) return interpolate(state,sc,sp,eye);
-		else return cached(state,sc,sp,eye);
+		if (lightcache->ready())
+			return interpolate(state,sc,sp,eye);
+		else
+			return cached(state,sc,sp,eye);
 	}
-	else return normalSample(state,sc,sp,eye);
+	else
+		return normalSample(state, sc, sp, eye);
 }
 
 
@@ -135,7 +138,11 @@ photonData_t *pathLight_t::getPhotonData(renderState_t &state)const
 color_t pathLight_t::getLight(renderState_t &state,const surfacePoint_t &sp,
 		const scene_t &sc,const vector3d_t &eye,photonData_t *data)const
 {
-	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
+	vector3d_t N;
+	if (ignorms && (!lightcache->ready()))
+		N = FACE_FORWARD(sp.Ng(), sp.Nd(), eye);
+	else
+		N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 	color_t total(0,0,0);
 	if(imap!=NULL)
 	{
@@ -197,8 +204,8 @@ static bool followCaustic(vector3d_t &ray,color_t &raycolor,
 	fresnel(edir,N,caus_IOR,kr,kt);
 	color_t ref=caus_rcolor*kr;
 	color_t trans=caus_tcolor*kt;
-	CFLOAT pref=ref.getRed()+ref.getGreen()+ref.getBlue();
-	CFLOAT ptrans=trans.getRed()+trans.getGreen()+trans.getBlue();
+	CFLOAT pref = ref.getR() + ref.getG() + ref.getB();
+	CFLOAT ptrans = trans.getR() + trans.getG() + trans.getB();
 	if( (pref==0.0) && (ptrans==0.0) ) return false;
 	if((pref/(pref+ptrans))>ourRandom())
 	{
@@ -213,9 +220,9 @@ static bool followCaustic(vector3d_t &ray,color_t &raycolor,
 	return true;
 }
 
-color_t pathLight_t::takeSample(renderState_t &state,const vector3d_t &N,
-		const surfacePoint_t &sp,const scene_t &sc,PFLOAT &avgD,PFLOAT &minD,
-		bool caching)const
+color_t pathLight_t::takeSample(renderState_t &state, const vector3d_t &N,
+		const surfacePoint_t &sp, const scene_t &sc, PFLOAT &avgD, PFLOAT &minD,
+		bool caching) const
 {
 	PFLOAT maxdist=1000000*sc.getWorldResolution()*sp.Z();
 	int oldlevel=state.rayDivision;
@@ -239,45 +246,104 @@ color_t pathLight_t::takeSample(renderState_t &state,const vector3d_t &N,
 	for(int ite=1;ite>0;--ite,repetitions+=1.0)
 	{
 		sampler->reset();
-		for(int i=0;i<samples;++i)
-		{
-			color_t raycolor(1.0);
-			vector3d_t ray = sampler->nextDirection(sp.P(),N, sp.NU(), sp.NV(), i, 0,raycolor);
-			vector3d_t startray = ray;
-			point3d_t where = sp.P();
-			if(caching) HNUM+=1;
-			state.skipelement=sp.getOrigin();
-			for(int j=0,cj=0;j<maxdepth;++j)
-			{
-				if (raycolor.energy()<0.05) break;
-				surfacePoint_t tempsp;
-				if (!sc.firstHit(state,tempsp, where, ray, true)) //background reached
+
+		if (occmode) {
+			// occlusion mode
+			color_t tcol(1.0);
+			surfacePoint_t tempsp;
+			state.skipelement = sp.getOrigin();
+			if (caching) {
+				for (int sm=0;sm<samples;sm++)
 				{
-					color_t contri=(startray*N)*raycolor*sc.getBackground(ray,state);
+					HNUM += 1;
+					vector3d_t dir = sampler->nextDirection(sp.P(), N, sp.NU(), sp.NV(), sm, 0, tcol);
+					if (occ_maxdistance>0) {
+						// distance limited mode
+						// normal unbiased hittest (no distance limit) to record correct mean harmdist.
+						bool bghit = true;
+						if (sc.firstHit(state, tempsp, sp.P(), dir, true))
+						{
+							if (tempsp.Z()>0) HD += 1.0/tempsp.Z();
+							if ((tempsp.Z()<M) || (M==0)) M = tempsp.Z();
+							bghit = false;
+						}
+						// except that if z>maxdistance, assume background hit as well
+						if (bghit || (tempsp.Z()>occ_maxdistance)) {
+							color_t contri(sc.getBackground(dir, state, true) * fabs(dir*N));
+							total += contri;
+							if (first) subtotal[sm & 3] += contri;
+						}
+					}
+					else {
+						// normal mode (unlimited distance, hemilight)
+						if (!sc.firstHit(state, tempsp, sp.P(), dir, true))
+						{
+							color_t contri(sc.getBackground(dir, state, true) * fabs(dir*N));
+							total += contri;
+							if (first) subtotal[sm & 3] += contri;
+						}
+						else {
+							if (tempsp.Z()>0) HD += 1.0/tempsp.Z();
+							if ((tempsp.Z()<M) || (M==0)) M = tempsp.Z();
+						}
+					}
+				}
+			}
+			else {
+				for (int sm=0;sm<samples;sm++)
+				{
+					vector3d_t dir = sampler->nextDirection(sp.P(), N, sp.NU(), sp.NV(), sm, 0, tcol);
+					if (!((occ_maxdistance>0) ?
+								sc.isShadowed(state, sp, sp.P()+occ_maxdistance*dir) :
+								sc.isShadowed(state, sp, dir)))
+						total += sc.getBackground(dir, state, true) * fabs(dir*N);
+				}
+			}
+		}
+		else {
+			// full GI
+			for(int i=0;i<samples;++i)
+			{
+				color_t raycolor(1.0);
+				vector3d_t ray = sampler->nextDirection(sp.P(),N, sp.NU(), sp.NV(), i, 0,raycolor);
+				vector3d_t startray = ray;
+				point3d_t where = sp.P();
+				if(caching) HNUM+=1;
+				state.skipelement=sp.getOrigin();
+				for(int j=0,cj=0;j<maxdepth;++j)
+				{
+					if (raycolor.energy()<0.05) break;
+					surfacePoint_t tempsp;
+					if (!sc.firstHit(state,tempsp, where, ray, true)) //background reached
+					{
+						color_t contri=(startray*N)*raycolor*sc.getBackground(ray, state, true);
+						total += contri;
+						if(first) subtotal[i%4]+=contri;
+						break;
+					}
+
+					if(caching)
+					{
+						if((j==0) && (cj==0) && (tempsp.Z()>0)) HD+=1.0/tempsp.Z();
+						if((j==0) && (cj==0) && ((tempsp.Z()<M) || (M==0)) ) M=tempsp.Z();
+					}
+					color_t light = getLight(state,tempsp,sc,-ray,data);
+					color_t contri=(startray*N)*raycolor*light;
 					total += contri;
 					if(first) subtotal[i%4]+=contri;
-					break;
+					vector3d_t NN;
+					if (ignorms && caching) NN=tempsp.Nd(); else NN=tempsp.N();
+					vector3d_t HN = FACE_FORWARD(tempsp.Ng(), NN, -ray);
+					if(!followCaustic(ray,raycolor, NN, HN, tempsp.getObject()))
+					{
+						raycolor *= tempsp.getShader()->getDiffuse(state, tempsp, -ray);
+						ray = sampler->nextDirection(tempsp.P(),HN, tempsp.NU(), tempsp.NV(),
+								i, j+1,raycolor);
+					}
+					else if(cj<maxcausdepth) {cj++;j--;}
+					where = tempsp.P();
+					state.skipelement=tempsp.getOrigin();
 				}
-				
-				if(caching)
-				{
-					if((j==0) && (cj==0) && (tempsp.Z()>0)) HD+=1.0/tempsp.Z();
-					if((j==0) && (cj==0) && ((tempsp.Z()<M) || (M==0)) ) M=tempsp.Z();
-				}
-				color_t light = getLight(state,tempsp,sc,-ray,data);
-				color_t contri=(startray*N)*raycolor*light;
-				total += contri;
-				if(first) subtotal[i%4]+=contri;
-				vector3d_t HN = FACE_FORWARD(tempsp.Ng(), tempsp.N(), -ray);
-				if(!followCaustic(ray,raycolor,tempsp.N(),HN,tempsp.getObject()))
-				{
-					raycolor *= tempsp.getShader()->getDiffuse(state, tempsp, -ray);
-					ray = sampler->nextDirection(tempsp.P(),HN, tempsp.NU(), tempsp.NV(),
-							i, j+1,raycolor);
-				}
-				else if(cj<maxcausdepth) {cj++;j--;}
-				where = tempsp.P();
-				state.skipelement=tempsp.getOrigin();
 			}
 		}
 		if(first)
@@ -288,12 +354,12 @@ color_t pathLight_t::takeSample(renderState_t &state,const vector3d_t &N,
 			for(int i=0;i<4;++i)
 			{
 				subtotal[i]*=sampler->multiplier();
-				if(subtotal[i].getRed()  >maxR) maxR=subtotal[i].getRed();
-				if(subtotal[i].getGreen()>maxG) maxG=subtotal[i].getGreen();
-				if(subtotal[i].getBlue() >maxB) maxB=subtotal[i].getBlue();
-				if(subtotal[i].getRed()  <minR) minR=subtotal[i].getRed();
-				if(subtotal[i].getGreen()<minG) minG=subtotal[i].getGreen();
-				if(subtotal[i].getBlue() <minB) minB=subtotal[i].getBlue();
+				if (subtotal[i].getR()>maxR) maxR = subtotal[i].getR();
+				if (subtotal[i].getG()>maxG) maxG = subtotal[i].getG();
+				if (subtotal[i].getB()>maxB) maxB = subtotal[i].getB();
+				if (subtotal[i].getR()<minR) minR = subtotal[i].getR();
+				if (subtotal[i].getG()<minG) minG = subtotal[i].getG();
+				if (subtotal[i].getB()<minB) minB = subtotal[i].getB();
 			}
 			color_t min(minR,minG,minB),max(maxR,maxG,maxB);
 			sc.adjustColor(min);
@@ -319,13 +385,13 @@ color_t pathLight_t::takeSample(renderState_t &state,const vector3d_t &N,
 	total*=sampler->multiplier()/repetitions;
 	return total;
 }
-		
 
 color_t pathLight_t::normalSample(renderState_t &state,const scene_t &sc, 
 		const surfacePoint_t sp, const vector3d_t &eye) const
 {
 	color_t light;
-	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
+	vector3d_t N; 
+	N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 	const shader_t *sha = sp.getShader();
 	color_t total(0.0);
 
@@ -468,7 +534,11 @@ color_t pathLight_t::interpolate(renderState_t &state,const scene_t &s,
 		const surfacePoint_t sp, const vector3d_t &eye) const
 {
 	if (sp.getShader()->getDiffuse(state, sp, eye).energy()<0.05f) return color_t(0.0);
-	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
+	vector3d_t N;
+	if (ignorms)
+		N = FACE_FORWARD(sp.Ng(), sp.Nd(), eye);
+	else
+		N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 	//point3d_t pP=toRealPolar(sp.P(),s);
 	point3d_t pP=lightcache->toPolar(sp.P(),state);
 	
@@ -516,6 +586,7 @@ color_t pathLight_t::interpolate(renderState_t &state,const scene_t &s,
 		// Antialiasing , more samples needed
 		cout<<".";cout.flush();
 		PFLOAT H,M;
+		if (ignorms) N = FACE_FORWARD(sp.Ng(), sp.Nd(), eye);
 		color_t ncol=takeSample(state,N,sp,s,H,M,true);
 		proxy->addSample(state,lightSample_t(N,ncol,H, sp.P(),
 			            lightcache->toPolar(sp.P(),state),M,state.traveled*s.getWorldResolution(),1.0));
@@ -584,12 +655,12 @@ bool pathLight_t::testRefinement(const scene_t &sc)
 				5,pathLight_t::weightNoDist,weightLimit);
 		for(vector<foundSample_t>::iterator j=samples.begin();j!=samples.end();++j)
 		{
-			if(j->S->mixed.getRed()  >maxR) maxR=j->S->mixed.getRed();
-			if(j->S->mixed.getGreen()>maxG) maxG=j->S->mixed.getGreen();
-			if(j->S->mixed.getBlue() >maxB) maxB=j->S->mixed.getBlue();
-			if(j->S->mixed.getRed()  <minR) minR=j->S->mixed.getRed();
-			if(j->S->mixed.getGreen()<minG) minG=j->S->mixed.getGreen();
-			if(j->S->mixed.getBlue() <minB) minB=j->S->mixed.getBlue();
+			if (j->S->mixed.getR()>maxR) maxR = j->S->mixed.getR();
+			if (j->S->mixed.getG()>maxG) maxG = j->S->mixed.getG();
+			if (j->S->mixed.getB()>maxB) maxB = j->S->mixed.getB();
+			if (j->S->mixed.getR()<minR) minR = j->S->mixed.getR();
+			if (j->S->mixed.getG()<minG) minG = j->S->mixed.getG();
+			if (j->S->mixed.getB()<minB) minB = j->S->mixed.getB();
 		}
 		
 		color_t min(minR,minG,minB),max(maxR,maxG,maxB);
@@ -616,7 +687,11 @@ color_t pathLight_t::cached(renderState_t &state,const scene_t &sc,
 {
 	if (sp.getShader()->getDiffuse(state, sp, eye).energy()<0.05f) return color_t(0.0);
 
-	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
+	vector3d_t N;
+	if (ignorms && (!lightcache->ready()))
+		N = FACE_FORWARD(sp.Ng(), sp.Nd(), eye);
+	else
+		N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 	PFLOAT rq=1.0/(state.raylevel+1);
 	color_t total(0,0,0);
 	if(!lightcache->enoughFor(sp.P(),N,state,pathLight_t::weightNoPrec,desiredWeight*rq))
@@ -686,6 +761,8 @@ light_t *pathLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	bool show_samples=false;
 	bool useg=false;
 	PFLOAT cache_size = 0.01,angt=0.2,shadt=0.3;
+	// new param. switch to ignore displaced normals when building cache
+	bool ignorms = false;	// default is old situation: do use normals (results in dense bad sample distrib.)
 
 	params.getParam("power", power);
 	params.getParam("depth", depth);
@@ -698,6 +775,14 @@ light_t *pathLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	params.getParam("threshold",thr);
 	params.getParam("max_refinement",ref);
 
+	// new mode parameter
+	string _mode = ":)";
+	const string *mode=&_mode;
+	params.getParam("mode", mode);
+	bool occmode = (*mode=="occlusion");
+	PFLOAT occdist = -1;
+	params.getParam("maxdistance", occdist);
+
 	if (samples<1) {
 		WARNING << "Samples value too low, minimum is one\n";
 		samples = 1;
@@ -711,11 +796,12 @@ light_t *pathLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 		params.getParam("recalculate",recalculate);
 		params.getParam("show_samples",show_samples);
 		params.getParam("gradient",useg);
+		params.getParam("ignore_bumpnormals", ignorms);
 		if(search<3) search=3;
 		//render.repeatFirstPass();
 	}
 	pathLight_t *path=new pathLight_t(samples, power, depth,cdepth, useqmc,
-			cache,cache_size,thr,recalculate,direct,show_samples,grid,ref);
+			cache,cache_size,thr,recalculate,direct,show_samples,grid,ref, occmode, occdist, ignorms);
 	if(cache) path->setCacheThreshold(shadt,search);
 	return path;
 }
