@@ -63,7 +63,8 @@ class pathLight_t : public light_t
 	public:
 		pathLight_t(int nsam, CFLOAT pwr, int depth, int cdepth,bool uQ,
 				bool ca=false,PFLOAT casiz=1.0,CFLOAT thr=0.1,bool recal=true,
-				bool di=false, bool shows=false,int grid=36,int ref=2);
+				bool di=false, bool shows=false,int grid=36,int ref=2,
+				bool _occmode=false, PFLOAT occdist=-1, bool _ignorms=false);
 
 		void setCacheThreshold(PFLOAT s,int se) 
 		{
@@ -134,6 +135,10 @@ class pathLight_t : public light_t
 		const globalPhotonLight_t::irHash_t *irhash;
 		photonData_t *photonData;
 		CFLOAT threshold,devaluated,desiredWeight,weightLimit;
+		bool occmode;
+		PFLOAT occ_maxdistance;
+		bool ignorms;
+
 		std::vector<foundSample_t> stsamples;
 		cacheProxy_t *_proxy;
 };
