@@ -21,6 +21,8 @@
  */
 
 #include "photonlight.h"
+#include "spectrum.h"
+
 using namespace std;
 #include <algorithm>
 
@@ -68,7 +70,8 @@ photonLight_t::photonLight_t(const point3d_t &f,const point3d_t _to,PFLOAT angle
 	dangle=angle;
 	K=search;
 	color=c;
-	pow=inte*(angle/180.0);
+	//pow=inte*(angle/180.0);
+	pow = inte * 2.0*M_PI*(1.0-angle_cos);
 	fixedRadius=fr;
 	cluster=clus;
 	if(mod==DIFFUSE)
@@ -89,60 +92,107 @@ photonLight_t::photonLight_t(const point3d_t &f,const point3d_t _to,PFLOAT angle
 	HSEQ = NULL;
 	use_QMC = useqmc;
 	if (use_QMC) {
-	  int base=2, md=maxdepth*2+1; // first shot Hammersly
-	  HSEQ = new Halton[md];
-	  for (int i=0;i<md;i++) {
-	    HSEQ[i].setBase(base);
-	    base = nextPrime(base);
-	  }
+		int base=2, md=(maxdepth+1)*2;
+		HSEQ = new Halton[md];
+		for (int i=0;i<md;i++) {
+			HSEQ[i].setBase(base);
+			base = nextPrime(base);
+		}
 	}
 	use_in_indirect=false;
 }
 
-void photonLight_t::shoot_photon_caustic(scene_t &scene,photon_t &photon,
-		const vector3d_t &dir,PFLOAT dis)
+void photonLight_t::shoot_photon_caustic(scene_t &scene, photon_t &photon,
+					const vector3d_t &dir, PFLOAT dis)
 {
-	if(depth>maxdepth) return;
+	if (depth>maxdepth) return;
 	depth++;
 	surfacePoint_t sp;
-	if(!scene.firstHit(nullstate,sp,photon.position(),dir)) {depth--;return;}
-	dis+=sp.Z();
-	const void *oldorigin=nullstate.skipelement;
-	nullstate.skipelement=sp.getOrigin();
-	
-	if(! (sp.getObject())->caustics() )
+	if (!scene.firstHit(nullstate, sp, photon.position(), dir)) { depth--;  return; }
+	dis += sp.Z();
+	const void *oldorigin = nullstate.skipelement;
+	nullstate.skipelement = sp.getOrigin();
+
+	const object3d_t* obj = sp.getObject();
+	const shader_t* sha = sp.getShader();
+	color_t caus_rcolor, caus_tcolor;
+	PFLOAT caus_IOR;
+	// try to get caustics colors and ior from shader first
+	// (which really should be done in the first place anyway, simplified here, so textures are still not taken into account)
+	// if not available, use params from object
+	bool caustics = sha->getCaustics(nullstate, sp, dir, caus_rcolor, caus_tcolor, caus_IOR);
+	if (!caustics)
 	{
-		if(depth>1)
+		caustics = obj->caustics();
+		caus_rcolor = obj->caus_rcolor;
+		caus_tcolor = obj->caus_tcolor;
+		caus_IOR = obj->caus_IOR;
+	}
+
+	// for caustics, using pure random instead of QMC seq. looks better in this case
+	if ((!caustics) || (ourRandom()<sha->getDiffuse(nullstate, sp, dir).energy()))
+	{
+		if (depth>1)
 		{
-			photon.position(sp.P(),bias);
-			insert(*hash,photonMark_t(photon));
+			photon.position(sp.P(), bias);
+			insert(*hash, photonMark_t(photon));
 			stored++;
 		}
 	}
 	else
 	{
-		photon.position(sp.P(),bias);
-		vector3d_t edir=photon.lastPosition()-photon.position();
+		photon.position(sp.P(), bias);
+		vector3d_t edir = photon.lastPosition()-photon.position();
 		edir.normalize();
-		vector3d_t N=FACE_FORWARD(sp.Ng(),sp.N(),edir);
-		CFLOAT kr,kt;
-		fresnel(edir,sp.N(),(sp.getObject())->caus_IOR,kr,kt);
+		vector3d_t N = FACE_FORWARD(sp.Ng(),sp.N(),edir);
+		CFLOAT kr, kt;
+		fresnel(edir, sp.N(), caus_IOR, kr, kt);
 
-		if( !((sp.getObject())->caus_rcolor.null() ) )
+		bool dorefl=(!caus_rcolor.null());
+		if (dorefl)
 		{
-			vector3d_t newdir=reflect(N,edir);
-			photon_t rphoton=photon;
-			rphoton.filter( kr * (sp.getObject())->caus_rcolor );
-			shoot_photon_caustic(scene,rphoton,newdir,dis);
+			vector3d_t newdir = reflect(N,edir);
+			photon_t rphoton = photon;
+			rphoton.filter(caus_rcolor*kr);
+			shoot_photon_caustic(scene, rphoton, newdir, dis);
 		}
-		if( !((sp.getObject())->caus_tcolor.null() ) )
+		if (!caus_tcolor.null())
 		{
-			vector3d_t newdir=refract(sp.N(),edir,(sp.getObject())->caus_IOR);
-			if(!newdir.null())
+			vector3d_t newdir;
+			PFLOAT disp_pw, cyA, cyB;
+			color_t beer;
+			sha->getDispersion(disp_pw, cyA, cyB, beer);
+			if (nullstate.chromatic && (disp_pw>0.0))
 			{
-				photon_t tphoton=photon;
-				tphoton.filter( kt * (sp.getObject())->caus_tcolor );
-				shoot_photon_caustic(scene,tphoton,newdir,dis);
+				color_t dcol(1.0);
+				// instead of totally randomly selecting wavelength, just use current photon number
+				nullstate.cur_ior = getIORcolor(((CFLOAT)emitted+ourRandom())/(CFLOAT)Np, cyA, cyB, dcol);
+				newdir = refract(sp.N(), edir, nullstate.cur_ior);
+				nullstate.chromatic = false;
+				if (!newdir.null())
+				{
+					photon_t tphoton = photon;
+					tphoton.filter(caus_tcolor*dcol*kt);
+					shoot_photon_caustic(scene, tphoton, newdir, dis);
+				}
+			}
+			else {
+				if (disp_pw>0.0)
+					newdir = refract(sp.N(), edir, nullstate.cur_ior);
+				else
+					newdir = refract(sp.N(), edir, caus_IOR);
+				if (!newdir.null())
+				{
+					photon_t tphoton = photon;
+					color_t ctc(caus_tcolor);
+					if ((disp_pw>0.0) && (!beer.null())) {
+						color_t be(-sp.Z()*beer);
+						be.set(exp(be.getR()), exp(be.getG()), exp(be.getB()));
+						ctc *= be;
+					}
+					tphoton.filter(ctc*kt);
+					shoot_photon_caustic(scene, tphoton, newdir, dis);
+				}
 			}
 		}
 	}
@@ -179,12 +229,10 @@ void photonLight_t::shoot_photon_diffuse(scene_t &scene,photon_t &photon,
 		energy_t ene(edir,photon.color());
 		PFLOAT r1, r2;
 		if (use_QMC) {
- 			int d2 = depth<<1;	// first shot Ham. (onedim.)
- 			r1=HSEQ[d2-1].getNext();  r2=HSEQ[d2].getNext();
+ 			int d2 = (depth<<1);
+ 			r1=HSEQ[d2].getNext();  r2=HSEQ[d2+1].getNext();
 		}
 		else { r1=ourRandom();  r2=ourRandom(); }
-		//vector3d_t refDir = randomVectorCone(Ng,0.05,r1,r2);
-		//vector3d_t refDir = HemiVec_COSW(Ng, sp.NU(), sp.NV(), r1, r2);
  		vector3d_t refDir = randomVectorCone(Ng, sp.NU(), sp.NV(), 0.05, r1, r2);
 		color_t newcolor=sha->fromRadiosity(nullstate,sp,ene,refDir);
 		photon.color(newcolor);
@@ -249,82 +297,100 @@ struct pointCross_f
 color_t photonLight_t::illuminate(renderState_t &state,const scene_t &s,const surfacePoint_t sp,
 															const vector3d_t &eye)const
 {
-	color_t total(0.0);
-	if(!sp.getObject()->reciveRadiosity()) return total;
-	//photonQueue_t found;
-	vector<foundPhoton_t> found;
-	compareFound_f cfound;
-	foundPhoton_t temp;
-	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye),VX,VY;
-	if(mode==DIFFUSE) createCS(N,VX,VY);
+	if(!sp.getObject()->reciveRadiosity()) return color_t(0, 0, 0);
+	// Allocate an vector that will be used as a heap with
+	// space for K photons. We are looking for the closest K
+	// photons, so the exact size is known.
+	vector<foundPhoton_t> found(0);
+	found.reserve(K);
+	vector3d_t N = FACE_FORWARD(sp.Ng(), sp.N(), eye);
 	gObjectIterator_t<photonMark_t *,point3d_t,pointCross_f> ite(tree,sp.P());
 	for(;!ite;++ite)
 	{
-		temp.dis=(sp.P()-(*ite)->position()).length();
-		if((temp.dis<=fixedRadius) && (((*ite)->direction()*N)>0.0))
+		// First check if the direction of the photon is appropriate
+		// for the current surface position. This is quite cheap, so
+		// do it before testing if the photon is in the search radius.
+		// However, this test is not only for early rejection but 
+		// also a required property for a gathered photon.
+		if (((*ite)->direction()*N)<=0.0)
+		{
+			continue;
+		}
+		// Now check if the photon is so close that it should be put
+		// in the heap.
+		const PFLOAT dis = (sp.P()-(*ite)->position()).length();		
+		if (dis<=fixedRadius)
 		{
-			temp.p=*ite;
-
 			if(found.size()==K)
 			{
-				//if(temp.dis<found.top().dis)
-				if(temp.dis<found.front().dis)
+				if(dis<found.front().dis)
 				{
-					found.push_back(temp);
-					push_heap(found.begin(),found.end(),cfound);
-					pop_heap(found.begin(),found.end(),cfound);
-					found.pop_back();
-						//found.push(temp);
-						//found.pop();
+					// Swap the most distant photon at the end.
+					pop_heap(found.begin(),found.end(), cfound);
+					// Override it with the current photon.
+					found.back().p = *ite;
+					found.back().dis = dis;
+					// Heap the new photon into the heap.
+					push_heap(found.begin(),found.end(), cfound);
 				}
 			}
 			else
 			{
+				// Put the photon in the vector and heap it.
+				foundPhoton_t temp;
+				temp.p = *ite;
+				temp.dis = dis;
 				found.push_back(temp);
-				push_heap(found.begin(),found.end(),cfound);
-				//found.push(temp);
+				push_heap(found.begin(),found.end(), cfound);
 			}
 		}
 	}
 
 	if(found.size()<2) return color_t(0,0,0);
-	PFLOAT area;
-	PFLOAT radius=found.front().dis;
-	//PFLOAT radius=found.top().dis;
-	PFLOAT minarea=4*M_PI/(PFLOAT)Np;
-	CFLOAT filter;
+
+	const PFLOAT radius=found.front().dis;
+	color_t total(0.0);
+
 	const shader_t *sha= sp.getShader();
 	vector<sample_t> samples(0);
-	if(mode==DIFFUSE) samples.reserve(found.size());
-	//while(!found.empty())
+	vector3d_t VX,VY;
+	if(mode==DIFFUSE)
+	{
+		createCS(N,VX,VY);
+		samples.reserve(found.size());
+	}
+
 	for(vector<foundPhoton_t>::iterator i=found.begin();i!=found.end();++i)
 	{
-		//const photonMark_t *p=found.top().p;
 		const photonMark_t *p=i->p;
-		//PFLOAT dis=found.top().dis;
-		PFLOAT dis=i->dis;
-		filter=filterCone(dis,radius);
-		energy_t ene(p->direction(),p->color()*filter);
-		total=total+sha->fromLight(state,sp,ene,eye);
+		const PFLOAT dis=i->dis;
+		const CFLOAT filter=filterCone(dis,radius);
+		const energy_t ene(p->direction(),p->color()*filter);
+		total += sha->fromLight(state,sp,ene,eye);
 		if(mode==DIFFUSE)
 		{
-			sample_t sample;
 			vector3d_t sub=p->position()-sp.P();
 			sub/=radius;
+			sample_t sample;
 			sample.x=sub*VX;
 			sample.y=sub*VY;
 			sample.weight=(p->direction()*N)*filter*0.5;
 			samples.push_back(sample);
 		}
-		//found.pop();
 	}
-	area=M_PI*radius*radius;
-	const PFLOAT SQRT2=sqrt(2.0);
-	if(mode==DIFFUSE) area*=fillArea(samples,SQRT2);
-	
-	if(area<minarea) area=minarea;
-	CFLOAT factor=4*M_PI/((CFLOAT)Np*area);
-	total=total*factor;
+	PFLOAT area = M_PI*radius*radius;
+	if(mode==DIFFUSE)
+	{
+		const PFLOAT SQRT2=1.414213562373095;
+		area*=fillArea(samples,SQRT2);
+	}	
+	const PFLOAT minarea=4.0*M_PI/(PFLOAT)Np;
+	if(area<minarea)
+	{
+		area=minarea;
+	}
+	const CFLOAT factor=4.0*M_PI/((CFLOAT)Np*area);
+	total *= factor;
 	return total;
 }
 
@@ -406,10 +472,11 @@ void photonLight_t::init(scene_t &scene)
 	{
 		photon_t photon(color*pow,from);
 		PFLOAT r1, r2;
-		if (use_QMC) { r1=HSEQ[0].getNext();  r2=((PFLOAT)i+0.5)/(PFLOAT)Np; }
+		if (use_QMC) { r1=HSEQ[0].getNext();  r2=HSEQ[1].getNext(); }
 		else { r1=ourRandom();  r2=ourRandom(); }
 		dir = randomVectorCone(light_dir, LU, LV, angle_cos, r1, r2);
 		if (dir.null()) continue;
+		nullstate.chromatic = true;
 		if (mode==CAUSTIC) shoot_photon_caustic(scene, photon, dir);
 		if (mode==DIFFUSE) shoot_photon_diffuse(scene, photon, dir);
 		emitted++;
