@@ -4,8 +4,13 @@ using namespace std;
 
 __BEGIN_YAFRAY
 
-softLight_t::softLight_t(const point3d_t &f,const color_t &c,CFLOAT p,int resol,int radius,GFLOAT biass)
+softLight_t::softLight_t(const point3d_t &f, const color_t &c, CFLOAT p,
+			int resol, int radius, GFLOAT biass, CFLOAT gli, CFLOAT glo, int glt)
 {
+	glow_int = gli;
+	glow_ofs = glo;
+	glow_type = glt;
+
 	cube[0].set(0.0,-1.0,0.0);
 	cube[1].set(1.0,0.0,0.0);
 	cube[2].set(0.0,1.0,0.0);
@@ -156,20 +161,22 @@ void softLight_t::init(scene_t &scene)
 color_t softLight_t::illuminate(renderState_t &state,const scene_t &s,const surfacePoint_t sp,
 																const vector3d_t &eye)const
 {
-	vector3d_t L=from-sp.P();
-	vector3d_t dir=L;
+	vector3d_t L = from-sp.P();
+	vector3d_t dir = L;
 	dir.normalize();
-	vector3d_t shadowdir=sp.P()-from;
+	vector3d_t shadowdir = sp.P()-from;
 	shadowdir.normalize();
 	const shader_t *sha= sp.getShader();
-	GFLOAT x,y;
-	int si=guessSide(shadowdir,x,y);
-	
-	int ix=(int)x,iy=(int)y;
-	CFLOAT shadow=mixShadow(si,ix-R,iy-R,ix+R,iy+R,x,y,
-			L.length());
-	energy_t ene(dir, (shadow*pow*color)/(L*L));
-	return sha->fromLight(state,sp,ene,eye);
+	GFLOAT x, y;
+	int si = guessSide(shadowdir,x,y);
+	int ix=(int)x, iy=(int)y;
+	CFLOAT shadow = mixShadow(si, ix-R, iy-R, ix+R, iy+R, x, y, L.length());
+	const color_t lcol(shadow*pow*color);
+	energy_t ene(dir, lcol/(L*L));
+	color_t col = sha->fromLight(state, sp, ene, eye);
+	// not shadowed
+	if (glow_int>0) col += glow_int * (pow*color) * getGlow(from, sp, eye, glow_ofs, glow_type);
+	return col;
 }
 
 void softLight_t::fillSide(int s,const vector3d_t &corner,
@@ -233,15 +240,22 @@ light_t *softLight_t::factory(paramMap_t &params,renderEnvironment_t &render)
 	CFLOAT power=1.0;
 	GFLOAT bias=0.1;
 	int res=100,radius=1;
-	
-	params.getParam("from",from);
-	params.getParam("color",color);
-	params.getParam("power",power);
-	params.getParam("res",res);
-	params.getParam("radius",radius);
-	params.getParam("bias",bias);
 
-	return new softLight_t(from,color,power,res,radius,bias);
+	params.getParam("from", from);
+	params.getParam("color", color);
+	params.getParam("power", power);
+	params.getParam("res", res);
+	params.getParam("radius", radius);
+	params.getParam("bias", bias);
+
+	// glow params
+	CFLOAT gli=0, glo=0;
+	int glt=0;
+	params.getParam("glow_intensity", gli);
+	params.getParam("glow_type", glt);
+	params.getParam("glow_offset", glo);
+
+	return new softLight_t(from, color, power, res, radius, bias, gli, glo, glt);
 }
 
 pluginInfo_t softLight_t::info()
