@@ -69,7 +69,6 @@ __BEGIN_YAFRAY
 #define A_RECVR   6
 #define A_CIOR    7
 
-
 #define A_POWER   8
 #define A_BIAS    9
 #define A_SAMPLES 10
@@ -92,7 +91,12 @@ __BEGIN_YAFRAY
 #define A_DOFDIST  64
 // camtype
 #define A_CAMTYPE  65
-
+// bokeh
+#define A_BOKEH		66
+#define A_BKHBIAS	67
+#define A_BKHROT	68
+// aspect ratio
+#define A_ASPECT	69
 
 #define A_ATTR   22
 #define A_FACTOR 23
@@ -609,16 +613,12 @@ lval_t join_object( std::vector<lval_t>::iterator v );
 class camera_data_t : public ast_t
 {
 	public:
-		camera_data_t() {id=AST_CAMERA;};
-		std::string name;
-		point3d_t from;
-		point3d_t to;
-		point3d_t up;
+		camera_data_t() { id=AST_CAMERA; }
+		std::string name, camtype, bkhtype, bkhbias;
+		point3d_t from, to, up;
 		int resx, resy;
-		GFLOAT dfocal, laperture;
+		GFLOAT dfocal, laperture, dofdist, bkh_ro, aspect;
 		bool dof_use_qmc;
-		GFLOAT dofdist;
-		std::string camtype;
 		virtual ~camera_data_t() {};
 };
 /*
