@@ -85,7 +85,7 @@ render_t::~render_t()
 void render_t::loadPlugins(const string &path)
 {
 	typedef void (reg_t)(renderEnvironment_t &);
-	cout<<"Loading plugins ..."<<endl;
+	cout<<"Loading plugins from '"<<path<<"'..."<<endl;
 	list<string> plugins=listDir(path);
 	
 	for(list<string>::iterator i=plugins.begin();i!=plugins.end();++i)
@@ -98,6 +98,8 @@ void render_t::loadPlugins(const string &path)
 		registerPlugin(*this);
 		pluginHandlers.push_back(plug);
 	}
+	if(pluginHandlers.size() == 0) cout<<"[WARNING]: ";
+	cout<<"found "<<pluginHandlers.size()<<" plugins!\n";
 }
 
 void * render_t::itemList(ast_t *ast)
@@ -721,11 +723,11 @@ void * render_t::mesh(ast_t *ast)
 	}
 	for(vector<triangle_t>::iterator i=faces.begin();i!=faces.end();++i)
 	{
-		long int n=(int)((*i).a);
+		long int n=(long int)((*i).a);
 		if((n>=(long int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
 		(*i).a=&(mesh->points->points)[n];
-		n=(int)((*i).b);
+		n=(long int)((*i).b);
 		if((n>=(long int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
 		(*i).b=&(mesh->points->points)[n];
