@@ -31,9 +31,9 @@ color_t genericShader_t::fromRadiosity(renderState_t &state,const surfacePoint_t
 																		const energy_t &ene,const vector3d_t &eye)const
 {
 	if( (FACE_FORWARD(sp.Ng(),sp.N(),eye) * ene.dir)<0) return color_t(0.0);
-	color_t lkc=scolor,lks=ks;
+	color_t lkc=scolor, lks=ks;
 	CFLOAT lkh=hard;
-	if( ! mods.empty() )
+	if (!mods.empty())
 	{
 		for(vector<modulator_t>::const_iterator ite=mods.begin();ite!=mods.end();++ite)
 		{
@@ -50,29 +50,30 @@ color_t genericShader_t::fromLight(renderState_t &state,const surfacePoint_t &sp
 	edir.normalize();
 	vector3d_t N=FACE_FORWARD(sp.Ng(),sp.N(),edir);
 	CFLOAT inte=N*energy.dir;
+	if (inte<0) return color_t(0.0);
 
 	color_t lkc=scolor,lks=ks;
 	CFLOAT lkh=hard;
 
-	if( ! mods.empty() )
+	if (!mods.empty())
 	{
 		for(vector<modulator_t>::const_iterator ite=mods.begin();ite!=mods.end();++ite)
 		{
 			(*ite).modulate(lkc,lks,lkh,sp, eye);
 		}
 	}
-	edir=reflect(N,edir);
+	edir = reflect(N,edir);
 	CFLOAT refle=edir*energy.dir;
-	if (refle<0) refle=0;
-	else refle=std::pow((CFLOAT)refle,(CFLOAT)lkh);
+	if (refle<0) refle=0; else refle=std::pow((CFLOAT)refle,(CFLOAT)lkh);
 
-	if(inte>0) return (edif*inte*energy.color*lkc) + lks*refle*energy.color;
-	else return color_t(0.0);
+	return (edif*inte*energy.color*lkc) + lks*refle*energy.color;
 }
 
 color_t genericShader_t::fromWorld(renderState_t &state,const surfacePoint_t &sp,const scene_t &s,
 															const vector3d_t &eye)const
 {
+	if (environment) return environment->stdoutColor(state, sp, eye, &s);
+
 	color_t Rresul(0.0), Tresul(0.0);
 
 	CFLOAT fKr,fKt;
@@ -82,6 +83,8 @@ color_t genericShader_t::fromWorld(renderState_t &state,const surfacePoint_t &sp
 	vector3d_t N=FACE_FORWARD(sp.Ng(),sp.N(),edir);
 	vector3d_t Ng=FACE_FORWARD(sp.Ng(),sp.Ng(),edir);
 	CFLOAT &cont=state.contribution;
+	bool &chroma = state.chromatic;
+	PFLOAT &cur_ior = state.cur_ior;
 	CFLOAT oldcont=cont;
 
 	if ((N*eye)<0) N=Ng;
@@ -125,22 +128,56 @@ color_t genericShader_t::fromWorld(renderState_t &state,const surfacePoint_t &sp
 	if (!cur_rfRcol.null())
 	{
 		vector3d_t ref;
-		ref=refract(sp.N(),edir,IOR);
-		if(ref.null() && tir) ref=reflect(N,edir);
-		if(!ref.null())
-		{
-			CFLOAT nt= (maxT<fKt ) ? maxT : fKt;
-			if((nt*cont)>0.01)
+		if (chroma && (dispersion_power>0.0)) {
+			Tresul.black();
+			color_t dispcol(1.0);
+			CFLOAT ds_scale=1.f/(PFLOAT)dispersion_samples;
+			for (int ds=0;ds<dispersion_samples;ds++) {
+				PFLOAT djt = dispersion_jitter ? ourRandom() : 0.5;
+				PFLOAT nior = getIORcolor((ds+djt)*ds_scale, CauchyA, CauchyB, dispcol);
+				ref = refract(sp.N(), edir, nior);
+				if (ref.null() && tir) ref = reflect(N, edir);
+				if (!ref.null())
+				{
+					CFLOAT nt = (1.0<fKt) ? 1.0 : fKt;
+					if ((nt*cont)>0.01)
+					{
+						cont *= nt;
+						chroma = false;
+						cur_ior = nior;
+						Tresul += dispcol * nt * s.raytrace(state, sp.P(), ref);
+						cont = oldcont;
+					}
+				}
+			}
+			Tresul *= ds_scale*cur_rfRcol;
+		}
+		else {
+			if (!chroma)
+				ref = refract(sp.N(), edir, cur_ior);
+			else
+				ref = refract(sp.N(), edir, IOR);
+			if (ref.null() && tir) ref = reflect(N, edir);
+			if (!ref.null())
 			{
-				cont *= nt;
-				color_t nrefr = cur_rfRcol*nt;
-				Tresul = nrefr*s.raytrace(state,sp.P(), ref);
-				cont = oldcont;
+				CFLOAT nt = (1.0<fKt) ? 1.0 : fKt;
+				if ((nt*cont)>0.01)
+				{
+					cont *= nt;
+					Tresul = cur_rfRcol * nt * s.raytrace(state, sp.P(), ref);
+					// absorption
+					if ((!beer_sigma_a.null()) && (state.raylevel>0)) {
+						color_t be(-sp.Z()*beer_sigma_a);
+						be.set(exp(be.getR()), exp(be.getG()), exp(be.getB()));
+						Tresul *= be;
+					}
+					cont = oldcont;
+				}
 			}
 		}
-	
 	}
-	if( ! mods.empty() )
+
+	if (!mods.empty())
 	{
 		for(vector<modulator_t>::const_iterator ite=mods.begin();ite!=mods.end();++ite)
 		{
@@ -186,6 +223,18 @@ shader_t * genericShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 	// new fresnel angle dependent colors
 	color_t reflected2(0.0), transmitted2(0.0);
 
+	// dispersion parameters
+	CFLOAT disp_pw = 0;
+	int disp_sam = 0;
+	bool disp_jit = false;
+	color_t beer(0.0);
+	bparams.getParam("dispersion_power", disp_pw);
+	bparams.getParam("dispersion_samples", disp_sam);
+	bparams.getParam("dispersion_jitter", disp_jit);
+
+	// transmit absorption coefficient
+	bparams.getParam("absorption", beer);
+
 	bparams.getParam("color", color);
 	bparams.getParam("specular", specular);
 	bparams.getParam("reflected", reflected);
@@ -202,9 +251,15 @@ shader_t * genericShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 	if (!bparams.getParam("reflected2", reflected2)) reflected2 = reflected;
 	if (!bparams.getParam("transmitted2", transmitted2)) transmitted2 = transmitted;
 
-	genericShader_t *ns=new genericShader_t(color,hard, 1.0, reflected, transmitted,
-						specular, reflected2, transmitted2,
-						minref, 1.0, ior, fast_fresnel,tir);
+	// environment shader, for use with other blocks, used in fromWorld()
+	string _env_name="";
+	const string *env_name=&_env_name;
+	bparams.getParam("environment", env_name);
+	shader_t* env = render.getShader(*env_name);
+
+	genericShader_t *ns=new genericShader_t(color,hard, (CFLOAT)1.0, reflected, transmitted,
+						specular, reflected2, transmitted2, env,
+						minref, (CFLOAT)1.0, ior, fast_fresnel, tir, disp_pw, disp_sam, disp_jit, beer);
 	for(list<paramMap_t>::iterator i=lmod.begin();i!=lmod.end();++i)
 	{
 		string _texname;
@@ -285,10 +340,13 @@ shader_t * genericShader_t::factory(paramMap_t &bparams,std::list<paramMap_t> &l
 		modu.ofsY(ofsy);
 		modu.ofsZ(ofsz);
 		if (size!=1.0) modu.size(size);
+		/*
 		if (*mode=="mix") modu.mode(MIX);
 		if (*mode=="mul") modu.mode(MUL);
 		if (*mode=="add") modu.mode(ADD);
 		if (*mode=="sub") modu.mode(SUB);
+		*/
+		modu.string2modetype(*mode);
 		modu.color(color);
 		modu.specular(specular);
 		modu.hard(hard);
