@@ -27,7 +27,7 @@
 #endif
 
 #include<iostream>
-#include<cmath>
+//#include<cmath>
 #include "vector3d.h"
 
 __BEGIN_YAFRAY
@@ -65,7 +65,7 @@ protected:
 inline matrix4x4_t  operator * (const matrix4x4_t &a,const matrix4x4_t &b)
 {
 	matrix4x4_t aux;
-
+	
 	for(int i=0;i<4;i++)
 		for(int k=0;k<4;k++)
 		{
@@ -76,24 +76,18 @@ inline matrix4x4_t  operator * (const matrix4x4_t &a,const matrix4x4_t &b)
 	return aux;
 }
 
-inline vector3d_t  operator * (const matrix4x4_t &a,const vector3d_t &b)
+inline vector3d_t  operator * (const matrix4x4_t &a, const vector3d_t &b)
 {
-	vector3d_t aux;
-
-	aux.x=a[0][0]*b.x+a[0][1]*b.y+a[0][2]*b.z;
-	aux.y=a[1][0]*b.x+a[1][1]*b.y+a[1][2]*b.z;
-	aux.z=a[2][0]*b.x+a[2][1]*b.y+a[2][2]*b.z;
-	return aux;
+	return vector3d_t(a[0][0]*b.x+a[0][1]*b.y+a[0][2]*b.z,
+										a[1][0]*b.x+a[1][1]*b.y+a[1][2]*b.z,
+										a[2][0]*b.x+a[2][1]*b.y+a[2][2]*b.z);
 }
 
-inline point3d_t  operator * (const matrix4x4_t &a,const point3d_t &b)
+inline point3d_t  operator * (const matrix4x4_t &a, const point3d_t &b)
 {
-	point3d_t aux;
-
-	aux.x=a[0][0]*b.x+a[0][1]*b.y+a[0][2]*b.z+a[0][3];
-	aux.y=a[1][0]*b.x+a[1][1]*b.y+a[1][2]*b.z+a[1][3];
-	aux.z=a[2][0]*b.x+a[2][1]*b.y+a[2][2]*b.z+a[2][3];
-	return aux;
+	return  point3d_t(a[0][0]*b.x+a[0][1]*b.y+a[0][2]*b.z+a[0][3],
+										a[1][0]*b.x+a[1][1]*b.y+a[1][2]*b.z+a[1][3],
+										a[2][0]*b.x+a[2][1]*b.y+a[2][2]*b.z+a[2][3]);
 }
 //matrix4x4_t rayToZ(const point3d_t &from,const vector3d_t & ray);
 YAFRAYCORE_EXPORT std::ostream & operator << (std::ostream &out,matrix4x4_t &m);
