@@ -147,13 +147,20 @@ meshObject_t::meshObject_t(const vector<point3d_t> &ver, const vector<vector3d_t
 		i->vcol=convertIterator(faces_vcol,fvcol,i->vcol);
 	}
 		
-	vector<triangle_t *> *ltri=new vector<triangle_t *>(ts.size());
-	for(vector<triangle_t>::iterator i=triangles.begin();i!=triangles.end();++i)
-		(*ltri)[i-triangles.begin()]=&(*i);
+//	vector<triangle_t *> *ltri=new vector<triangle_t *>(ts.size());
+//	for(vector<triangle_t>::iterator i=triangles.begin();i!=triangles.end();++i)
+//		(*ltri)[i-triangles.begin()]=&(*i);
 
 	//tree=buildGenericTree(ltri,face_calc_bound,face_is_in_bound,face_get_pos,10);
-	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
-	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
+//	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
+//	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
+	
+	// Lynx ->
+	const triangle_t **tris=new const triangle_t*[triangles.size()];
+	for(unsigned int i=0;i<triangles.size();++i)
+		tris[i] = &(triangles[i]);
+	n_tree = new kdTree_t(tris, triangles.size(), -1, -1, 1.2, 0.40 );
+	delete[] tris;
 }
 
 meshObject_t::meshObject_t(bool _hasorco, const matrix4x4_t &M, const vector<point3d_t> &ver, const vector<vector3d_t> &nor,
@@ -187,6 +194,7 @@ meshObject_t::meshObject_t(bool _hasorco, const matrix4x4_t &M, const vector<poi
 	}
 
 	tree=NULL;
+	n_tree=0;
 	transform(M);
 }
 
@@ -408,6 +416,7 @@ meshObject_t::~meshObject_t()
 	//cout<<"rays "<<rays<<" hitted "<<hitted<<endl;
 	//cout<<"avgsize "<<((float)leafst/(float)leafs)<<endl;
 	if (tree!=NULL) delete tree;
+	if(n_tree) delete n_tree;
 }
 
 void meshObject_t::transform(const matrix4x4_t &m)
@@ -459,16 +468,24 @@ void meshObject_t::transform(const matrix4x4_t &m)
 		(*ite).recNormal();
 	unt=false;
 
-	vector<triangle_t *> *ltri=new vector<triangle_t *>(triangles.size());
-	for(vector<triangle_t>::iterator i=triangles.begin();
-			i!=triangles.end();++i)
-		(*ltri)[i-triangles.begin()]=&(*i);
-	if(tree!=NULL) delete tree;
+//	vector<triangle_t *> *ltri=new vector<triangle_t *>(triangles.size());
+//	for(vector<triangle_t>::iterator i=triangles.begin();
+//			i!=triangles.end();++i)
+//		(*ltri)[i-triangles.begin()]=&(*i);
+//	if(tree!=NULL) delete tree;
 	//tree=buildGenericTree(ltri,face_calc_bound,face_is_in_bound,face_get_pos,4);
-	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
-	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
+//	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
+//	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
 	recalcBound();
-
+	
+	
+	// Lynx ->
+	const triangle_t **tris=new const triangle_t*[triangles.size()];
+	for(unsigned int i=0;i<triangles.size();++i)
+		tris[i] = &(triangles[i]);
+	if(n_tree != 0) delete n_tree;
+	n_tree = new kdTree_t(tris, triangles.size(), -1, -1, 1.2, 0.40 );
+	
 	// backOrco, replace translation with (transformed!) bound center
 	bound.get(p1, p2);
 	mv = toVector(0.5*(p1+p2));
@@ -510,24 +527,15 @@ bool meshObject_t::shoot(renderState_t &state,surfacePoint_t &where,
 	//mray.ray=ray;
 	PFLOAT minZ=-1;
 	if(dis<0) dis=numeric_limits<PFLOAT>::infinity();
-/*
-	if(shadow && (lasto==(object3d_t *)this) && (lastb!=NULL))
-	{
-		vector<triangle_t *>::const_iterator i=lastb->begin();
-		vector<triangle_t *>::const_iterator end=lastb->end();
-		for(;i!=end;++i)
-		{
-			if( (*i)->hit(from,ray))
-			{
-				PFLOAT Z=(*i)->intersect(from,ray);
-				if((Z>0) && ((Z<dis) || (dis<0))) return true;
-			}
-		}
-	}
-	*/
-	//int c=0;
-	//gObjectIterator_t<triangle_t *,mray_t,rayCross_f> i(tree,mray);
-	//geomeIterator_t<vector<triangle_t *> > ite(tree,dis,from,ray,shadow);
+
+	//Lynx
+	bool isec;
+	PFLOAT Z=dis;
+	if(shadow) return n_tree->IntersectS(from, ray, dis, &hitt);
+	else isec = n_tree->Intersect(from, ray, dis, &hitt, Z);
+
+	if(!isec) return false;
+	/*
 	pureBspIterator_t<vector<triangle_t *> > ite(tree,dis,bound,from,ray,shadow);
 	for(;!ite;++ite)
 	{
@@ -557,9 +565,10 @@ bool meshObject_t::shoot(renderState_t &state,surfacePoint_t &where,
 		}
 	}
 	if(shadow) return false;
-	if(hitt==NULL) return false;
-	point3d_t h=from+minZ*ray;
-	surfacePoint_t temp=hitt->getSurface(h,minZ,hasorco);
+	*/
+	if(hitt==NULL){cout << "d'oh!"; return false; }
+	point3d_t h=from+/*min*/Z*ray;
+	surfacePoint_t temp=hitt->getSurface(h,/*min*/Z,hasorco);
 	temp.setObject((object3d_t *)this);
 	temp.setOrigin(hitt);
 	if(temp.getShader()==NULL) temp.setShader(shader);
@@ -761,7 +770,7 @@ bool triBoxOverlap(const bound_t &bound,const point3d_t &tria,
   point3d_t boxcenter(bound.centerX(),bound.centerY(),bound.centerZ());
   point3d_t tmp1,tmp2;
   bound.get(tmp1,tmp2);
-  point3d_t boxhalfsize(tmp2.x-boxcenter.x,tmp2.y-boxcenter.y,tmp2.z-boxcenter.z);
+  point3d_t boxhalfsize(0.51*(tmp2.x-tmp1.x)+0.00001,0.51*(tmp2.y-tmp1.y)+0.00001,0.51*(tmp2.z-tmp1.z)+0.00001);
   /*    use separating axis theorem to test overlap between triangle and box */
   /*    need to test for overlap in these directions: */
   /*    1) the {x,y,z}-directions (actually, since we use the AABB of the triangle */
