@@ -39,7 +39,7 @@ int pcount;
 
 renderState_t::renderState_t() :raylevel(0),depth(0),contribution(1.0),/*lastobject(NULL)
 	,lastobjectelement(NULL),*/ skipelement(NULL),currentPass(0),rayDivision(1),traveled(0)
-	,pixelNumber(0) 
+	,pixelNumber(0), chromatic(true), cur_ior(1)
 {
 }
 
@@ -62,6 +62,8 @@ scene_t::scene_t()
 	repeatFirst=false;
 	scymin=scxmin=-2;
 	scymax=scxmax=2;
+	alpha_maskbackground = alpha_premultiply = false;
+	clamp_rgb = false;
 }
 
 scene_t::~scene_t()
@@ -195,12 +197,8 @@ void scene_t::fog_addToCol(CFLOAT depth, color_t &curcol) const
 	}
 }
 
-color_t scene_t::raytrace(renderState_t &state,const point3d_t &from,
-		const vector3d_t & ray)const
+color_t scene_t::raytrace(renderState_t &state, const point3d_t &from, const vector3d_t & ray)const
 {
-	// jump to alternative raytrace for indirect sampling if indirect_samples set
-	if (indirect_samples)
-		return raytrace_indir(state,from, ray);
 	int &l_raylevel=state.raylevel;
 	CFLOAT &l_depth=state.depth;
 	++l_raylevel;
@@ -238,20 +236,22 @@ color_t scene_t::raytrace(renderState_t &state,const point3d_t &from,
 	{
 		vector3d_t eye = from-sp.P();	// also now needed in displace
 		eye.normalize();
-		PFLOAT oldtraveled=state.traveled;
-		state.traveled+=sp.Z();
+		PFLOAT oldtraveled = state.traveled;
+		state.traveled += sp.Z();
 		sp.getShader()->displace(state, sp, eye, world_resolution);
 		color_t res=light(state,sp,from);
 		l_raylevel--;
-		l_depth=(from-sp.P()).length();
+		l_depth = (from-sp.P()).length();
 		// add simple fog if enabled
 		fog_addToCol(l_depth, res);
-		state.traveled=oldtraveled;
+		state.traveled = oldtraveled;
 		return res;
 	}
 	l_raylevel--;
 	l_depth=-1;
-	color_t bg = getBackground(ray,state);
+	// don't include background if alpha_maskbackground flag set (only primary rays)
+	color_t bg(0.0);
+	if (!(alpha_maskbackground && (l_raylevel!=0))) bg = getBackground(ray, state);
 	// add simple fog if enabled
 	fog_addToCol(l_depth, bg);
 	return bg;
@@ -276,92 +276,6 @@ color_t scene_t::light(renderState_t &state,const surfacePoint_t &sp,const point
 		return flights;
 }
 
-
-color_t scene_t::raytrace_indir(renderState_t &state,const point3d_t &from,
-		const vector3d_t & ray, bool isdif) const
-{
-	int &l_raylevel=state.raylevel;
-	CFLOAT &l_depth=state.depth;
-	++l_raylevel;
-	if(l_raylevel>=maxraylevel)
-	{
-		l_raylevel--;
-		l_depth=-1;
-		return color_t(0,0,0);
-	}
-	point3d_t f=from+ray*min_raydis;
-	surfacePoint_t sp,temp;
-	bool found=false;
-	//for(objectIterator_t ite(*BTree,f,ray);!ite;ite++)
-	for(geomeIterator_t<object3d_t> ite(BTree,numeric_limits<PFLOAT>::infinity(),f,ray);!ite;ite++)
-	{
-		if((*ite)->shoot(state,temp,f,ray))
-		{
-			if(temp.Z()>0.0)
-			{
-				ite.limit(temp.Z());
-				if(!found)
-				{
-					sp=temp;
-					found=true;
-				}
-				else if( temp.Z()<sp.Z()  ) sp=temp;
-			}
-		}
-	}
-	if(found)
-	{
-		// need to set screen position in calculated surfacepoint here for possible win texmap.
-		sp.setScreenPos(state.screenpos);
-		const shader_t *sha= sp.getShader();
-		if(sha==NULL)
-		{
-			l_raylevel--;
-			return color_t(0,0,0);
-		}
-		vector3d_t eye=from-sp.P();
-		eye.normalize();
-		PFLOAT oldtraveled=state.traveled;
-		state.traveled+=sp.Z();
-		sp.getShader()->displace(state, sp, eye, world_resolution);
-		// indirect light pass
-		color_t flights(0.0);
-		color_t basecol = sha->getDiffuse(state, sp, eye);
-		if ((!isdif) && (indirect_samples) && (basecol.energy()>0.05)) {
-			vector3d_t nm = FACE_FORWARD(sp.Ng(), sp.N(), eye);
-			color_t radcol(0.0);
-			for (int difx=0;difx<indirect_grid;difx++) {
-			  for (int dify=0;dify<indirect_grid;dify++) {
-			    PFLOAT z1=((PFLOAT)difx+ourRandom())*indirect_gdiv,
-			           z2=((PFLOAT)dify+ourRandom())*indirect_gdiv2pi;
-			    radcol += raytrace_indir(state,sp.P(), (sp.NU()*cos(z2) + 
-								sp.NV()*sin(z2))*sqrt(1.0-z1) + nm*sqrt(z1), true);
-			  }
-			}
-			flights += basecol*radcol*indirect_power;
-		}
-		for(list<light_t *>::const_iterator ite=light_list.begin();
-				ite!=light_list.end();++ite)
-		{
-			flights += (*ite)->illuminate(state,*this,sp,eye);
-		}
-		flights += sha->fromWorld(state,sp,*this,from-sp.P());
-		l_raylevel--;
-		l_depth=(from-sp.P()).length();
-		// add simple fog if enabled
-		fog_addToCol(l_depth, flights);
-		state.traveled=oldtraveled;
-		return flights;
-	}
-	l_raylevel--;
-	l_depth=-1;
-	color_t bg = getBackground(ray,state);
-	// add simple fog if enabled
-	fog_addToCol(l_depth, bg);
-	return bg;
-}
-
-
 void scene_t::setupLights()
 {
 	fprintf(stderr,"Setting up lights ...\n");
@@ -498,7 +412,7 @@ bool scene_t::doOnePass(renderState_t &state,
 			state.pixelNumber=j+numline*colorBuffer.resx();
 			for (int ms=0;ms<AA_minsamples;ms++) {
 			  globalpass = cursam = (pass-1)*AA_minsamples + ms;
-			  fx = AA_pixelwidth*(fast_vdC(cursam) - 0.5);
+			  fx = AA_pixelwidth*(RI_vdC(cursam) - 0.5);
 			  fy = AA_pixelwidth*((cursam+0.5)*totsamdiv - 0.5);
 				// only used with 'win' texture mapping
 				screenpos.set(2.0*(((PFLOAT)j+fx)/colorBuffer.resx())-1.0, 1.0-2.0*(((PFLOAT)numline+fy)/colorBuffer.resy()), 0);
@@ -641,6 +555,7 @@ void scene_t::render(colorOutput_t &out)
 	setupLights();
 
 	cout<<endl;
+	
 	while(repeatFirst)
 	{
 		cout<<"\rFake   pass: [";
@@ -805,101 +720,136 @@ bool scene_t::firstHit(renderState_t &state,surfacePoint_t &sp,const point3d_t &
 	return found;
 }
 
-void scene_t::render(renderArea_t &area)const
+
+void scene_t::render(renderArea_t &area) const
 {
 	renderState_t state;
 	CFLOAT &contri=state.contribution;
 	CFLOAT &pdep=state.depth;
+	bool &chroma = state.chromatic;
+	PFLOAT &cur_ior = state.cur_ior;
 	int &globalpass=state.currentPass;
 	int resx=render_camera->resX();
 	int resy=render_camera->resY();
 	// now includes alpha channel as well, no separate calculation
 	colorA_t fcol;
 
+	PFLOAT fx=0.5, fy=0.5;
+
 	//First pass
+	unsigned int sc1=0, sc2=0;
+	PFLOAT wt;
 	for(int i=area.Y;i<(area.Y+area.H);++i)
 		for(int j=area.X;j<(area.X+area.W);++j)
 		{
-			state.raylevel=-1;
-			state.screenpos.set(2.0*((PFLOAT)j/(PFLOAT)resx)-1.0, 
-					1.0-2.0*((PFLOAT)i/(PFLOAT)resy), 0);
-			vector3d_t ray=render_camera->shootRay(j, i);
-			contri = 1.0;
-			globalpass=0;
-			state.pixelNumber=j+i*resx;
+			if (AA_jitterfirst && (AA_passes!=0)) {
+				fx = RI_vdC(++sc1);
+				fy = RI_S(++sc2);
+			}
+			state.screenpos.set(2.0*(((PFLOAT)j+fx)/(PFLOAT)resx)-1.0, 
+					1.0-2.0*(((PFLOAT)i+fy)/(PFLOAT)resy), 0);
 			if ((state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) && 
 					(state.screenpos.y>=scymin) && (state.screenpos.y<scymax))
 			{
-				fcol = raytrace(state,render_camera->position(),ray);
-				fcol.expgam_Adjust(exposure, gamma_R);
-				if (pdep>=0) fcol.setAlpha(1.0); else fcol.setAlpha(0.0);
-				area.imagePixel(j,i) = fcol;
-				area.depthPixel(j,i) = pdep;
+				state.raylevel = -1;
+				vector3d_t ray = render_camera->shootRay((PFLOAT)j+fx, (PFLOAT)i+fy, wt);
+				contri = 1.0;
+				globalpass = 0;
+				state.pixelNumber = j+i*resx;
+				if (wt!=0.0) {
+					chroma = true;
+					cur_ior = 1.0;
+					fcol = raytrace(state, render_camera->position(), ray);
+					if (do_tonemap) fcol.expgam_Adjust(exposure, gamma_R, clamp_rgb);
+					if (pdep>=0) fcol.setAlpha(1.0); else fcol.setAlpha(0.0);
+					area.imagePixel(j,i) = fcol;
+					area.depthPixel(j,i) = pdep;
+				}
+				else {
+					area.imagePixel(j,i) = color_t(0.0);
+					area.depthPixel(j,i) = numeric_limits<PFLOAT>::infinity();
+				}
 			}
+			else area.imagePixel(j,i)=colorA_t(0.0);
 		}
 
-	PFLOAT fx, fy, totsamdiv = AA_minsamples*AA_passes;
+	PFLOAT totsamdiv = AA_minsamples*AA_passes;
 	if (totsamdiv!=0) totsamdiv = 1.0/totsamdiv;
-	for(int pass=0;pass<AA_passes;pass++)
+	for (int pass=0;pass<AA_passes;pass++)
 	{
 		area.checkResample(AA_threshold);
-		for(int i=area.Y;i<(area.Y+area.H);++i)
-			for(int j=area.X;j<(area.X+area.W);++j)
+		for (int i=area.Y;i<(area.Y+area.H);++i)
+			for (int j=area.X;j<(area.X+area.W);++j)
 			{
-				if(!area.resamplePixel(j,i)) continue;
+				if (!area.resamplePixel(j,i)) continue;
 				colorA_t totcol(0.0);
-				unsigned int cursam;
-				state.pixelNumber=j+i*resx;
-				
-				state.raylevel=-1;
+				state.pixelNumber = j+i*resx;
+				unsigned int cursam=0;
 
+				int totnumsam = 0;
 				for (int ms=0;ms<AA_minsamples;ms++) 
 				{
 					globalpass = cursam = pass*AA_minsamples + ms;
-					state.raylevel=-1;
-					fx = AA_pixelwidth*(fast_vdC(cursam) - 0.5);
-					fy = AA_pixelwidth*((cursam+0.5)*totsamdiv - 0.5);
+					state.raylevel = -1;
+					fx = 0.5 + AA_pixelwidth*(RI_LP(cursam+state.pixelNumber) - 0.5);
+					fy = 0.5 + AA_pixelwidth*(cursam*totsamdiv - 0.5);
+					//fx = 0.5 + AA_pixelwidth*(HSEQ1.getNext() - 0.5);
+					//fy = 0.5 + AA_pixelwidth*(HSEQ2.getNext() - 0.5);
 					state.screenpos.set(2.0*(((PFLOAT)j+fx)/(PFLOAT)resx)-1.0, 
 							1.0-2.0*(((PFLOAT)i+fy)/(PFLOAT)resy), 0);
-					vector3d_t ray = render_camera->shootRay((PFLOAT)j+fx, (PFLOAT)i+fy);
-					if ((state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) &&
+					vector3d_t ray = render_camera->shootRay((PFLOAT)j+fx, (PFLOAT)i+fy, wt);
+					if ((wt!=0.0) && (state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) &&
 							(state.screenpos.y>=scymin) && (state.screenpos.y<scymax))
 					{
+						chroma = true;
+						cur_ior = 1.0;
 						fcol = raytrace(state,render_camera->position(), ray);
-						fcol.expgam_Adjust(exposure, gamma_R);
-						if (pdep>=0) fcol.setAlpha(fcol.getAlpha()+1.0);
+						if (do_tonemap) fcol.expgam_Adjust(exposure, gamma_R, clamp_rgb);
+						if (pdep>=0) fcol.setAlpha(1.0); else fcol.setAlpha(0.0);
 						totcol += fcol;
+						totnumsam++;
 					}
-					
 				}
-				CFLOAT mf = (CFLOAT)(pass*AA_minsamples+1);
-				area.imagePixel(j,i) = (mf*area.imagePixel(j,i) + totcol) / (mf+(CFLOAT)AA_minsamples);
+				CFLOAT mf = (CFLOAT)(pass*totnumsam+1);
+				area.imagePixel(j,i) = (mf*area.imagePixel(j,i) + totcol) / (mf+(CFLOAT)totnumsam);
 			}
 	}
+
+	if (alpha_premultiply) {
+	for (int i=area.Y;i<(area.Y+area.H);++i)
+		for (int j=area.X;j<(area.X+area.W);++j)
+			area.imagePixel(j, i).alphaPremultiply();
+	}
 }
 
 void scene_t::fakeRender(renderArea_t &area)const
 {
 	renderState_t state;
 	CFLOAT &contri=state.contribution;
+	bool &chroma = state.chromatic;
+	PFLOAT &cur_ior = state.cur_ior;
 	int &globalpass=state.currentPass;
 	int resx=render_camera->resX();
 	int resy=render_camera->resY();
 
 	//First pass
+	PFLOAT wt;
 	for(int i=area.Y;i<(area.Y+area.H);++i)
 		for(int j=area.X;j<(area.X+area.W);++j)
 		{
-			state.raylevel=-1;
-			state.screenpos.set(2.0*((PFLOAT)j/(PFLOAT)resx)-1.0, 
-					1.0-2.0*((PFLOAT)i/(PFLOAT)resy), 0);
-			vector3d_t ray=render_camera->shootRay(j, i);
+			state.raylevel = -1;
+			state.screenpos.set(2.0*(((PFLOAT)j+0.5)/(PFLOAT)resx)-1.0, 
+					1.0-2.0*(((PFLOAT)i+0.5)/(PFLOAT)resy), 0);
+			vector3d_t ray = render_camera->shootRay((PFLOAT)j+0.5, (PFLOAT)i+0.5, wt);
 			contri = 1.0;
-			globalpass=0;
-			state.pixelNumber=j+i*resx;
-			if(!((state.screenpos.x<scxmin) || (state.screenpos.x>scxmax) || 
-					(state.screenpos.y<scymin) || (state.screenpos.y>scymax)))
-				area.imagePixel(j,i)=raytrace(state,render_camera->position(),ray);
+			globalpass = 0;
+			state.pixelNumber = j+i*resx;
+			chroma = true;
+			cur_ior = 1.0;
+			if ((wt!=0.0) && (state.screenpos.x>=scxmin) && (state.screenpos.x<scxmax) &&
+					(state.screenpos.y>=scymin) && (state.screenpos.y<scymax))
+				area.imagePixel(j, i) = raytrace(state, render_camera->position(), ray);
+			else area.imagePixel(j, i) = colorA_t(0.0);
 		}
 }
 
