@@ -66,13 +66,18 @@ class YAFRAYCORE_EXPORT surfacePoint_t
 				GFLOAT u, GFLOAT v, color_t vcol,
 				PFLOAT d,const shader_t *sha=NULL,
 				bool uv=false, bool hvcol=false,bool horco=false)
-		{	suP=p;  suN=n;  suNg=g;
+		{
+			suP=p;  suN=n;  suNg=g;
 			suU=u;  suV=v;  vtxcol=vcol;
 			suZ=d;  obj=o; shader=sha;  hasuv=uv;  has_vcol=hvcol;
-			hasorco=horco;
-			orcoP=orc;
-			createCS(suN,suNU,suNV);
-			dudu=dudv=dvdu=dvdv=0;
+			hasorco = horco;
+			orcoP = orc;
+			createCS(suN, suNU, suNV);
+			// tangents
+			suTU = suNU;
+			suTV = suNV;
+			suNd = n;	// unmodified normal (not displaced)
+			dudu = dudv = dvdu = dvdv = 0;
 			originelement=NULL;
 		}
 
@@ -84,6 +89,9 @@ class YAFRAYCORE_EXPORT surfacePoint_t
 		/// Returns the normal.
 		const vector3d_t &N() const { return suN; }
 		vector3d_t &N() { return suN; }
+		/// Returns the unmodified normal (copy of N before displacement)
+		const vector3d_t &Nd() const { return suNd; }
+		vector3d_t &Nd() { return suNd; }
 		/// Returns the geometric normal.
 		const vector3d_t &Ng() const { return suNg; }
 		vector3d_t &Ng() { return suNg; }
@@ -134,6 +142,13 @@ class YAFRAYCORE_EXPORT surfacePoint_t
 		void setGradient(GFLOAT uu, GFLOAT uv, GFLOAT vu, GFLOAT vv)
 		{ dudu=uu;  dudv=uv;  dvdu=vu;  dvdv=vv; }
 
+		// tangent vectors, from uv, TV crossp of N and tu
+		void setTangent(const vector3d_t &tu) { suTU=tu;  suTV=suN^tu; }
+		const vector3d_t & TU() const { return suTU; }
+		vector3d_t & TU() { return suTU; }
+		const vector3d_t & TV() const { return suTV; }
+		vector3d_t & TV() { return suTV; }
+
 		// only used with 'win' texture coord. mode
 		void setScreenPos(const point3d_t &p) { screenpos=p; }
 		void getScreenPos(PFLOAT &sx, PFLOAT &sy) const { sx=screenpos.x;  sy=screenpos.y; }
@@ -142,7 +157,7 @@ class YAFRAYCORE_EXPORT surfacePoint_t
 		const void *getOrigin()const {return originelement;};
 	protected:
 		/// The surface normal
-		vector3d_t suN,suNU,suNV;
+		vector3d_t suN, suNU, suNV, suTU, suTV, suNd;
 		/// The geometric normal (not smoothed)
 		vector3d_t suNg;
 		/// surface texture coords
