@@ -47,7 +47,8 @@ class YAFRAYCORE_EXPORT triangle_t
 		bool Z_hit();
 		bool hit(const point3d_t &from,const vector3d_t &ray)
 		{
-			vector3d_t va=(*a)-from,vb=(*b)-from,vc=(*c)-from,r;
+			const vector3d_t va=(*a)-from,vb=(*b)-from,vc=(*c)-from;
+			vector3d_t r;
 			if((ray*normal)<0) r=-ray;
 			else r=ray;
 			if( ((va^vb)*r)<0 ) return false;
@@ -96,6 +97,7 @@ class YAFRAYCORE_EXPORT triangle_t
 
 		point3d_t *a,*b,*c;
 		vector3d_t *na,*nb,*nc;
+		vector3d_t *ta, *tb, *tc;
 		std::vector<GFLOAT>::iterator uv;
 		std::vector<CFLOAT>::iterator vcol;
 		bool hasuv, has_vcol;
